@@ -23,35 +23,36 @@ done
 
 if test "x${RESET}" = x1 ; then rm -fr ${BASELINE}/*.d4m ; fi
 
+mkdir ./results_test_meta
 for f in ${F} ; do
     echo "checking: $f"
-    if ! ${VG} ${execdir}/test_meta ${DMRTESTFILES}/${f}.dmr ./results/${f} ; then
-        failure "${execdir}/test_meta ${DMRTESTFILES}/${f}.dmr ./results/${f}"
+    if ! ${VG} ${execdir}/test_meta ${DMRTESTFILES}/${f}.dmr ./results_test_meta/${f} ; then
+        failure "${execdir}/test_meta ${DMRTESTFILES}/${f}.dmr ./results_test_meta/${f}"
     fi
-    ${NCDUMP} -h ./results/${f} > ./results/${f}.d4m
+    ${NCDUMP} -h ./results_test_meta/${f} > ./results_test_meta/${f}.d4m
     if test "x${TEST}" = x1 ; then
-	if ! diff -wBb ${BASELINE}/${f}.d4m ./results/${f}.d4m ; then
-	    failure "diff -wBb ${BASELINE}/${f}.ncdump ./results/${f}.d4m"
+	if ! diff -wBb ${BASELINE}/${f}.d4m ./results_test_meta/${f}.d4m ; then
+	    failure "diff -wBb ${BASELINE}/${f}.ncdump ./results_test_meta/${f}.d4m"
 	fi
     elif test "x${RESET}" = x1 ; then
 	echo "${f}:" 
-	cp ./results/${f}.d4m ${BASELINE}/${f}.d4m
+	cp ./results_test_meta/${f}.d4m ${BASELINE}/${f}.d4m
     fi
 done
 
 if test "x${CDLDIFF}" = x1 ; then
   for f in $CDL ; do
-    echo "diff -wBb ${CDLTESTFILES}/${f}.cdl ./results/${f}.d4m"
+    echo "diff -wBb ${CDLTESTFILES}/${f}.cdl ./results_test_meta/${f}.d4m"
     rm -f ./tmp
     cat ${CDLTESTFILES}/${f}.cdl \
     cat >./tmp
-    echo diff -wBbu ./tmp ./results/${f}.d4m
-    if ! diff -wBbu ./tmp ./results/${f}.d4m ; then
+    echo diff -wBbu ./tmp ./results_test_meta/${f}.d4m
+    if ! diff -wBbu ./tmp ./results_test_meta/${f}.d4m ; then
 	failure "${f}" 
     fi
   done
 fi
-
+rm -rf ./results_test_meta
 finish
 
 
