@@ -66,6 +66,9 @@ typedef enum {VAR, DIM, ATT} NC_OBJ_T;
 #define X_LONG_MIN	X_INT_MIN
 #define X_LONG_MAX	X_INT_MAX
 #define X_UINT_MAX	4294967295U
+#define X_INT64_MIN	(-9223372036854775807LL-1LL)
+#define X_INT64_MAX	9223372036854775807LL
+#define X_UINT64_MAX	18446744073709551615ULL
 #ifdef WIN32 /* Windows, of course, has to be a *little* different. */
 #define X_FLOAT_MAX	3.402823466e+38f
 #else
@@ -99,13 +102,17 @@ typedef enum {VAR, DIM, ATT} NC_OBJ_T;
  * as the netCDF dimid. */
 #define NC_DIMID_ATT_NAME "_Netcdf4Dimid"
 
+/** This is the name of the class HDF5 dimension scale attribute. */
+#define HDF5_DIMSCALE_CLASS_ATT_NAME "CLASS"
+
+/** This is the name of the name HDF5 dimension scale attribute. */
+#define HDF5_DIMSCALE_NAME_ATT_NAME "NAME"
+
 /* Boolean type, to make the code easier to read */
 typedef enum {NC_FALSE = 0, NC_TRUE = 1} nc_bool_t;
 
-#ifdef ENABLE_FILEINFO
 /*Forward*/
 struct NCFILEINFO;
-#endif
 
 /* Generic doubly-linked list node */
 typedef struct NC_LIST_NODE
@@ -125,7 +132,7 @@ typedef struct NC_DIM_INFO
    nc_bool_t unlimited;         /* True if the dimension is unlimited */
    nc_bool_t extended;          /* True if the dimension needs to be extended */
    nc_bool_t too_long;          /* True if len is too big to fit in local size_t. */
-   hid_t hdf_dimscaleid;
+   hid_t hdf_dimscaleid;        /* Non-zero if a DIM_WITHOUT_VARIABLE dataset is in use (no coord var). */
    HDF5_OBJID_T hdf5_objid;
    struct NC_VAR_INFO *coord_var; /* The coord var, if it exists. */
 } NC_DIM_INFO_T;
@@ -148,7 +155,6 @@ typedef struct NC_ATT_INFO
 /* This is a struct to handle the var metadata. */
 typedef struct NC_VAR_INFO
 {
-   NC_LIST_NODE_T l;            /* Use generic doubly-linked list (must be first) */
    char *name;
    char *hdf5_name; /* used if different from name */
    int ndims;
@@ -179,9 +185,6 @@ typedef struct NC_VAR_INFO
    int deflate_level;
    nc_bool_t shuffle;           /* True if var has shuffle filter applied */
    nc_bool_t fletcher32;        /* True if var has fletcher32 filter applied */
-   nc_bool_t szip;              /* True if var has szip filter applied */
-   int options_mask;
-   int pixels_per_block;
    size_t chunk_cache_size, chunk_cache_nelems;
    float chunk_cache_preemption;
 #ifdef USE_HDF4
@@ -189,7 +192,11 @@ typedef struct NC_VAR_INFO
    int sdsid;
    int hdf4_data_type;
 #endif /* USE_HDF4 */
-   /* Stuff below for diskless data files. */
+   /* Stuff for arbitrary filters */
+   unsigned int filterid;
+   size_t nparams;
+   unsigned int* params;
+   /* Stuff for diskless data files. */
    void *diskless_data;
 } NC_VAR_INFO_T;
 
@@ -264,6 +271,12 @@ typedef struct NC_TYPE_INFO
    } u;                         /* Union of structs, for each type/class */
 } NC_TYPE_INFO_T;
 
+typedef struct NC_VAR_ARRAY_T {
+	size_t nalloc;		/* number allocated >= nelems */
+	size_t nelems;		/* length of the array */
+	NC_VAR_INFO_T **value;
+} NC_VAR_ARRAY_T;
+
 /* This holds information for one group. Groups reproduce with
  * parthenogenesis. */
 typedef struct NC_GRP_INFO
@@ -275,12 +288,11 @@ typedef struct NC_GRP_INFO
    struct NC_HDF5_FILE_INFO *nc4_info;
    struct NC_GRP_INFO *parent;
    struct NC_GRP_INFO *children;
-   NC_VAR_INFO_T *var;
+   NC_VAR_ARRAY_T vars;
    NC_DIM_INFO_T *dim;
    NC_ATT_INFO_T *att;
    NC_TYPE_INFO_T *type;
    int nvars;
-   int ndims;
    int natts;
 } NC_GRP_INFO_T;
 
@@ -306,9 +318,6 @@ typedef struct  NC_HDF5_FILE_INFO
 #endif
    int flags;
    int cmode;
-   int nvars;
-   int ndims;
-   int natts;
    nc_bool_t parallel;          /* True if file is open for parallel access */
    nc_bool_t redef;             /* True if redefining an existing file */
    int fill_mode;               /* Fill mode for vars - Unused internally currently */
@@ -322,9 +331,7 @@ typedef struct  NC_HDF5_FILE_INFO
    nc_bool_t hdf4;              /* True for HDF4 file */
    int sdid;
 #endif /* USE_HDF4 */
-#ifdef ENABLE_FILEINFO
    struct NCFILEINFO* fileinfo;
-#endif
 } NC_HDF5_FILE_INFO_T;
 
 
@@ -343,6 +350,7 @@ int nc4_convert_type(const void *src, void *dest,
 /* These functions do HDF5 things. */
 int rec_detach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid);
 int rec_reattach_scales(NC_GRP_INFO_T *grp, int dimid, hid_t dimscaleid);
+int delete_existing_dimscale_dataset(NC_GRP_INFO_T *grp, int dimid, NC_DIM_INFO_T *dim);
 int nc4_open_var_grp2(NC_GRP_INFO_T *grp, int varid, hid_t *dataset);
 int nc4_put_vara(NC *nc, int ncid, int varid, const size_t *startp,
 		 const size_t *countp, nc_type xtype, int is_long, void *op);
@@ -389,8 +397,8 @@ int nc4_type_free(NC_TYPE_INFO_T *type);
 
 /* These list functions add and delete vars, atts. */
 int nc4_nc4f_list_add(NC *nc, const char *path, int mode);
-int nc4_var_list_add(NC_VAR_INFO_T **list, NC_VAR_INFO_T **var);
-int nc4_var_list_del(NC_VAR_INFO_T **list, NC_VAR_INFO_T *var);
+int nc4_var_add(NC_VAR_INFO_T **var);
+int nc4_var_del(NC_VAR_INFO_T *var);
 int nc4_dim_list_add(NC_DIM_INFO_T **list, NC_DIM_INFO_T **dim);
 int nc4_dim_list_del(NC_DIM_INFO_T **list, NC_DIM_INFO_T *dim);
 int nc4_att_list_add(NC_ATT_INFO_T **list, NC_ATT_INFO_T **att);
@@ -452,12 +460,9 @@ For netcdf4 files, capture state information about the following:
 5. Per file: _NCProperties attribute
 */
 
-#ifdef ENABLE_FILEINFO
-
 #define NCPROPS "_NCProperties"
 #define NCPROPS_VERSION (1)
 #define NCPROPSSEP  '|'
-#define NCPROPS_LENGTH (8192)
 
 /* Currently used properties */
 #define NCPVERSION "version" /* Of the properties format */
@@ -475,7 +480,6 @@ struct NCFILEINFO {
         int version; /* 0 => not defined */
         char hdf5ver[NC_MAX_NAME+1];
         char netcdfver[NC_MAX_NAME+1];
-        char text[NCPROPS_LENGTH+1]; /* Value of the NCPROPS attribute */
     } propattr;
 };
 
@@ -484,11 +488,10 @@ extern struct NCPROPINFO globalpropinfo;
 extern int NC4_fileinfo_init(void); /*libsrc4/ncinfo.c*/
 extern int NC4_get_fileinfo(struct NC_HDF5_FILE_INFO* info, struct NCPROPINFO*); /*libsrc4/ncinfo.c*/
 extern int NC4_put_propattr(struct NC_HDF5_FILE_INFO* info); /*libsrc4/ncinfo.c*/
+extern int NC4_buildpropinfo(struct NCPROPINFO* info,char** propdatap);
 
-/* ENABLE_FILEINFO => ENABLE_NETCDF4 */
 extern int NC4_hdf5get_libversion(unsigned*,unsigned*,unsigned*);/*libsrc4/nc4hdf.c*/
 extern int NC4_hdf5get_superblock(struct NC_HDF5_FILE_INFO*, int*);/*libsrc4/nc4hdf.c*/
 extern int NC4_isnetcdf4(struct NC_HDF5_FILE_INFO*); /*libsrc4/nc4hdf.c*/
-#endif /*ENABLE_FILEINFO*/
 
 #endif /* _NETCDF4_ */
