@@ -29,7 +29,8 @@
 #define X_INT_MAX	2147483647
 
 /* Given a filename, check its magic number */
-#define MAGIC_NUMBER_LEN 4
+/* Change magic number size from 4 to 8 to be more precise for HDF5 */
+#define MAGIC_NUMBER_LEN 8
 #define MAGIC_HDF5_FILE 1
 #define MAGIC_HDF4_FILE 2
 #define MAGIC_CDF1_FILE 1 /* std classic format */
@@ -115,19 +116,20 @@ typedef struct NC_Dispatch NC_Dispatch;
 extern int NCDISPATCH_initialize(void);
 extern int NCDISPATCH_finalize(void);
 
-extern NC_Dispatch* NCSUBSTRATE_dispatch_table;
-extern int NCSUBSTRATE_initialize(void);
-extern int NCSUBSTRATE_finalize(void);
-
 extern NC_Dispatch* NC3_dispatch_table;
 extern int NC3_initialize(void);
 extern int NC3_finalize(void);
 
-#ifdef USE_DAP
+#ifdef ENABLE_DAP
 extern NC_Dispatch* NCD2_dispatch_table;
 extern int NCD2_initialize(void);
 extern int NCD2_finalize(void);
 #endif
+#ifdef ENABLE_DAP4
+extern NC_Dispatch* NCD4_dispatch_table;
+extern int NCD4_initialize(void);
+extern int NCD4_finalize(void);
+#endif
 
 #ifdef USE_PNETCDF
 extern NC_Dispatch* NCP_dispatch_table;
@@ -141,12 +143,6 @@ extern int NC4_initialize(void);
 extern int NC4_finalize(void);
 #endif
 
-#ifdef USE_DAP
-extern NC_Dispatch* NCD4_dispatch_table;
-extern int NCD4_initialize(void);
-extern int NCD4_finalize(void);
-#endif
-
 /* Vectors of ones and zeros */
 extern size_t nc_sizevector0[NC_MAX_VAR_DIMS];
 extern size_t nc_sizevector1[NC_MAX_VAR_DIMS];
@@ -250,7 +246,8 @@ int (*inq_var_all)(int ncid, int varid, char *name, nc_type *xtypep,
                int *shufflep, int *deflatep, int *deflate_levelp,
                int *fletcher32p, int *contiguousp, size_t *chunksizesp,
                int *no_fill, void *fill_valuep, int *endiannessp,
-	       int *options_maskp, int *pixels_per_blockp);
+	       unsigned int* idp, size_t* nparamsp, unsigned int* params
+              );
 
 int (*var_par_access)(int, int, int);
 
@@ -294,6 +291,7 @@ int (*def_var_fletcher32)(int, int, int);
 int (*def_var_chunking)(int, int, int, const size_t*);
 int (*def_var_fill)(int, int, int, const void*);
 int (*def_var_endian)(int, int, int);
+int (*def_var_filter)(int, int, unsigned int, size_t, const unsigned int*);
 int (*set_var_chunk_cache)(int, int, size_t, size_t, float);
 int (*get_var_chunk_cache)(int ncid, int varid, size_t *sizep, size_t *nelemsp, float *preemptionp);
 #endif /*USE_NETCDF4*/
@@ -319,12 +317,12 @@ typedef struct NCcommon {
 	struct NC_Dispatch* dispatch;
 	void* dispatchdata; /* per-protocol instance data */
 	char* path; /* as specified at open or create */
-	int   substrate; /* ncid for another protocol on which to build */
 } NCcommon;
 
 extern size_t NC_atomictypelen(nc_type xtype);
 extern char* NC_atomictypename(nc_type xtype);
 
+#ifdef OBSOLETE
 /* Provide a dispatch table overlay facility */
 extern int NC_dispatch_overlay(const NC_Dispatch* overlay,
                                         const NC_Dispatch* base,
@@ -333,36 +331,33 @@ extern int NC_dispatch_overlay(const NC_Dispatch* overlay,
 /* Get/set the override dispatch table */
 extern NC_Dispatch* NC_get_dispatch_override(void);
 extern void NC_set_dispatch_override(NC_Dispatch*);
+#endif
 
-/* Does the path look like a url? */
-extern int NC_testurl(const char* path);
-/* Return model (0 or 3 or 4) as specified by the url */
-extern int NC_urlmodel(const char* path);
+/* Return model as specified by the url, if any;
+   return a modified url suitable for passing to curl
+*/
+extern int NC_urlmodel(const char* path, int mode, char** newurl);
 
 /* allow access url parse and params without exposing nc_url.h */
 extern int NCDAP_urlparse(const char* s, void** dapurl);
 extern void NCDAP_urlfree(void* dapurl);
 extern const char* NCDAP_urllookup(void* dapurl, const char* param);
 
-/* Test for specific set of servers */
-#if defined(DLL_NETCDF) /* Defined when library is a DLL */
-# if defined(DLL_EXPORT) /* Define when building the library. */
-#  define MSC_NCDISPATCH_EXTRA __declspec(dllexport)
-# else
-#  define MSC_NCDISPATCH_EXTRA __declspec(dllimport)
-# endif
+#if defined(DLL_NETCDF)
+# if defined(DLL_EXPORT)
+#  define NCC_EXTRA __declspec(dllexport)
 #else
-#  define MSC_NCDISPATCH_EXTRA
+#  define NCC_EXTRA __declspec(dllimport)
+# endif
+NCC_EXTRA extern int nc__testurl(const char* path, char** basename);
+#else
+extern int
+ nc__testurl(const char* parth, char** basename);
 #endif
 
-#define NCD_EXTERNL MSC_NCDISPATCH_EXTRA extern
-
-NCD_EXTERNL char* NC_findtestserver(const char*, const char**);
-NCD_EXTERNL int nc_open_mem(const char*, int, size_t, void*, int*);
-NCD_EXTERNL int nc_finalize();
-
 /* Ping a specific server */
-extern int NCDAP_ping(const char*);
+extern int NCDAP2_ping(const char*);
+extern int NCDAP4_ping(const char*);
 
 /* Misc */
 
@@ -372,6 +367,15 @@ extern int NC_inq_recvar(int ncid, int varid, int* nrecdims, int* is_recdim);
 
 #define nullstring(s) (s==NULL?"(null)":s)
 
+
+#undef TRACECALLS
+#ifdef TRACECALLS
+#include <stdio.h>
+#define TRACE(fname) fprintf(stderr,"call: %s\n",#fname)
+#else
+#define TRACE(fname)
+#endif
+
 extern size_t NC_coord_zero[NC_MAX_VAR_DIMS];
 extern size_t NC_coord_one[NC_MAX_VAR_DIMS];
 
@@ -379,6 +383,26 @@ extern int NC_argc;
 extern char* NC_argv[];
 extern int NC_initialized;
 
-NCD_EXTERNL int nc_initialize();
+extern int nc_initialize();
+extern int nc_finalize();
+
+
+
+/**
+Certain functions are in the dispatch table,
+but not in the netcdf.h API. These need to
+be exposed for use in delegation such as
+in libdap2.
+*/
+extern int
+NCDISPATCH_inq_var_all(int ncid, int varid, char *name, nc_type *xtypep,
+               int *ndimsp, int *dimidsp, int *nattsp,
+               int *shufflep, int *deflatep, int *deflate_levelp,
+               int *fletcher32p, int *contiguousp, size_t *chunksizesp,
+               int *no_fill, void *fill_valuep, int *endiannessp,
+	       unsigned int* idp, size_t* nparamsp, unsigned int* paramsp
+               );
+extern int
+NCDISPATCH_get_att(int ncid, int varid, const char* name, void* value, nc_type t);
 
 #endif /* _DISPATCH_H */
