@@ -7,7 +7,7 @@
 #ifdef USE_PARALLEL
 #include "netcdf_par.h"
 #endif
-#include "ncdap.h"
+#include "dapincludes.h"
 #include "dapdump.h"
 #include "dceconstraints.h"
 
@@ -584,7 +584,7 @@ dumpslice(DCEslice* slice)
                 (unsigned long)slice->stride,
                 (unsigned long)slice->last);
     }
-    strcat(buf,tmp);
+    strlcat(buf,tmp,sizeof(buf));
     return strdup(tmp);
 }
 
