@@ -4,7 +4,7 @@
  *********************************************************************/
 
 
-#include "ncdap.h"
+#include "dapincludes.h"
 #include "dapodom.h"
 #include "dapdump.h"
 #include "ncd2dispatch.h"
@@ -821,7 +821,7 @@ fprintf(stderr,"\n");
 	    count = dcesegmentsize(segment,0,rank0); /* how many to read */
 	    internlen = interntypesize*count;
             /* Read the whole variable directly into memory.*/
-            ocstat = oc_data_readn(conn,currentcontent,dap_zero,count,internlen,memory->next);
+            ocstat = oc_data_readn(conn,currentcontent,NC_coord_zero,count,internlen,memory->next);
 	    /* bump memory pointer */
 	    memory->next += internlen;
             if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto done;}
@@ -878,7 +878,7 @@ fprintf(stderr,"\n");
             size_t internlen;
 	    count = dcesegmentsize(segment,0,rank0); /* how many to read */
 	    internlen = interntypesize*count;
-            ocstat = oc_data_readn(conn,currentcontent,dap_zero,count,internlen,memory->next);
+            ocstat = oc_data_readn(conn,currentcontent,NC_coord_zero,count,internlen,memory->next);
             if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto done;}
 	}
     }
