@@ -6,34 +6,43 @@
 
 /* For MinGW Build */
 
-
+#if HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <stdio.h>
+#include <assert.h>
+#include <stdlib.h>
+#include <errno.h>
+#include <string.h>
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 
 /* Windows platforms, including MinGW, Cygwin, Visual Studio */
 #if defined(_WIN32) || defined(_WIN64)
 #include <windows.h>
 #include <winbase.h>
 #include <io.h>
-#else
+#endif
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include <assert.h>
-#include <stdlib.h>
-#include <errno.h>
-
-#ifndef ENOERR
-#define ENOERR 0
+#ifndef NC_NOERR
+#define NC_NOERR 0
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <string.h>
-
 #ifndef HAVE_SSIZE_T
-#define ssize_t int
+typedef int ssize_t;
 #endif
 
 #ifndef SEEK_SET
@@ -99,6 +108,41 @@ static int ncio_spx_close(ncio *nciop, int doUnlink);
 #ifndef POSIXIO_DEFAULT_PAGESIZE
 #define POSIXIO_DEFAULT_PAGESIZE 4096
 #endif
+
+/*! Cross-platform file length.
+ *
+ * Some versions of Visual Studio are throwing errno 132
+ * when fstat is used on large files.  This function is
+ * an attempt to get around that.
+ *
+ * @par fd File Descriptor.
+ * @return -1 on error, length of file (in bytes) otherwise.
+ */
+static off_t nc_get_filelen(const int fd) {
+
+  off_t flen;
+
+#ifdef HAVE_FILE_LENGTH_I64
+  __int64 file_len = 0;
+  if ((file_len = _filelengthi64(fd)) < 0) {
+    return file_len;
+  }
+  flen = (off_t)file_len;
+
+#else
+  int res = 0;
+  struct stat sb;
+  if((res = fstat(fd,&sb)) <0)
+    return res;
+
+  flen = sb.st_size;
+#endif
+
+  return flen;
+
+}
+
+
 /*
  * What is the system pagesize?
  */
@@ -134,7 +178,8 @@ pagesize(void)
 static size_t
 blksize(int fd)
 {
-#if defined(HAVE_ST_BLKSIZE)
+#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
+#ifdef HAVE_SYS_STAT_H
 	struct stat sb;
 	if (fstat(fd, &sb) > -1)
 	{
@@ -143,6 +188,7 @@ blksize(int fd)
 		return 8192;
 	}
 	/* else, silent in the face of error */
+#endif
 #endif
 	return (size_t) 2 * pagesize();
 }
@@ -159,7 +205,7 @@ fgrow(const int fd, const off_t len)
 	if (fstat(fd, &sb) < 0)
 		return errno;
 	if (len < sb.st_size)
-		return ENOERR;
+		return NC_NOERR;
 	{
 	    const long dumb = 0;
 	    /* we don't use ftruncate() due to problem with FAT32 file systems */
@@ -174,7 +220,7 @@ fgrow(const int fd, const off_t len)
 	    if (lseek(fd, pos, SEEK_SET) < 0)
 		return errno;
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -186,13 +232,23 @@ fgrow(const int fd, const off_t len)
 static int
 fgrow2(const int fd, const off_t len)
 {
-	struct stat sb;
-	if (fstat(fd, &sb) < 0)
-		return errno;
-	if (len <= sb.st_size)
-		return ENOERR;
-	{
-	    const char dumb = 0;
+
+
+  /* There is a problem with fstat on Windows based systems
+     which manifests (so far) when Config RELEASE is built.
+     Use _filelengthi64 isntead.
+
+     See https://github.com/Unidata/netcdf-c/issues/188
+
+  */
+
+
+  off_t file_len = nc_get_filelen(fd);
+  if(file_len < 0) return errno;
+  if(len <= file_len)
+    return NC_NOERR;
+  {
+    const char dumb = 0;
 	    /* we don't use ftruncate() due to problem with FAT32 file systems */
 	    /* cache current position */
 	    const off_t pos = lseek(fd, 0, SEEK_CUR);
@@ -205,7 +261,7 @@ fgrow2(const int fd, const off_t len)
 	    if (lseek(fd, pos, SEEK_SET) < 0)
 		return errno;
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 /* End OS */
 /* Begin px */
@@ -261,7 +317,7 @@ px_pgout(ncio *const nciop,
 	    return errno;
 	*posp += extent;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /*! Read in a page of data.
@@ -282,9 +338,6 @@ px_pgin(ncio *const nciop,
 {
 	int status;
 	ssize_t nread;
-    size_t read_count = 0;
-    ssize_t bytes_xfered = 0;
-    void *p = vp;
 #ifdef X_ALIGN
 	assert(offset % X_ALIGN == 0);
 	assert(extent % X_ALIGN == 0);
@@ -332,7 +385,7 @@ px_pgin(ncio *const nciop,
 
     if(nread != (ssize_t)extent) {
       status = errno;
-      if( nread == -1 || (status != EINTR && status != ENOERR))
+      if( nread == -1 || (status != EINTR && status != NC_NOERR))
         return status;
       /* else it's okay we read less than asked for */
       (void) memset((char *)vp + nread, 0, (ssize_t)extent - nread);
@@ -341,7 +394,7 @@ px_pgin(ncio *const nciop,
     *nreadp = nread;
 	*posp += nread;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* This struct is for POSIX systems, with NC_SHARE not in effect. If
@@ -403,7 +456,7 @@ px_rel(ncio_px *const pxp, off_t offset, int rflags)
 	}
 	pxp->bf_refcount--;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* This function indicates the file region starting at offset may be
@@ -474,7 +527,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 		int rflags,
 		void **const vpp)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	const off_t blkoffset = _RNDDOWN(offset, (off_t)pxp->blksz);
 	off_t diff = (size_t)(offset - blkoffset);
@@ -517,7 +570,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 				 middle,
 				 &pxp->bf_cnt,
 				 &pxp->pos);
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			pxp->bf_extent = 2 * pxp->blksz;
 			pxp->bf_cnt += pxp->blksz;
@@ -552,7 +605,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->blksz,
 					pxp->bf_base,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 			pxp->bf_cnt -= pxp->blksz;
@@ -571,7 +624,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->blksz,
 					pxp->bf_base,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 		}
@@ -589,7 +642,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 				 middle,
 				 &pxp->bf_cnt,
 				 &pxp->pos);
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			pxp->bf_extent = 2 * pxp->blksz;
 			pxp->bf_cnt += pxp->blksz;
@@ -617,7 +670,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->bf_cnt - pxp->blksz,
 					middle,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 			pxp->bf_cnt = pxp->blksz;
@@ -636,7 +689,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 			 pxp->bf_base,
 			 &pxp->bf_cnt,
 			 &pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_offset = blkoffset;
 		if(upper_cnt != 0)
@@ -661,7 +714,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 			pxp->bf_cnt,
 			pxp->bf_base,
 			&pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_rflags = 0;
 	}
@@ -673,7 +726,7 @@ pgin:
 		 pxp->bf_base,
 		 &pxp->bf_cnt,
 		 &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 	 pxp->bf_offset = blkoffset;
 	 pxp->bf_extent = blkextent;
@@ -692,7 +745,7 @@ done:
 #else
     *vpp = (void *)((signed char*)pxp->bf_base + diff);
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* Request that the region (offset, extent) be made available through
@@ -749,7 +802,7 @@ px_double_buffer(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	void *src;
 	void *dest;
 
@@ -759,7 +812,7 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 #endif
 	status = px_get(nciop, pxp, to, nbytes, RGN_WRITE,
 			&dest);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(pxp->slave == NULL)
@@ -786,7 +839,7 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 	pxp->slave->pos = pxp->pos;
 	status = px_get(nciop, pxp->slave, from, nbytes, 0,
 			&src);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 	if(pxp->pos != pxp->slave->pos)
 	{
@@ -821,7 +874,7 @@ ncio_px_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	off_t lower;
 	off_t upper;
 	char *base;
@@ -829,7 +882,7 @@ ncio_px_move(ncio *const nciop, off_t to, off_t from,
 	size_t extent;
 
 	if(to == from)
-		return ENOERR; /* NOOP */
+		return NC_NOERR; /* NOOP */
 
 	if(fIsSet(rflags, RGN_WRITE) && !fIsSet(nciop->ioflags, NC_WRITE))
 		return EPERM; /* attempt to write readonly file */
@@ -871,7 +924,7 @@ if(to > from)
 
 			status = px_double_buffer(nciop, toh, frm,
 				 	loopextent, rflags) ;
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			remaining -= loopextent;
 
@@ -887,7 +940,7 @@ else
 
 			status = px_double_buffer(nciop, to, from,
 				 	loopextent, rflags) ;
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			remaining -= loopextent;
 
@@ -897,7 +950,7 @@ else
 			from += loopextent;
 		}
 }
-		return ENOERR;
+		return NC_NOERR;
 	}
 
 #if INSTRUMENT
@@ -906,7 +959,7 @@ fprintf(stderr, "\tncio_px_move small\n");
 	status = px_get(nciop, pxp, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
@@ -927,14 +980,14 @@ static int
 ncio_px_sync(ncio *const nciop)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(fIsSet(pxp->bf_rflags, RGN_MODIFIED))
 	{
 		assert(pxp->bf_refcount <= 0);
 		status = px_pgout(nciop, pxp->bf_offset,
 			pxp->bf_cnt,
 			pxp->bf_base, &pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_rflags = 0;
 	}
@@ -1028,7 +1081,7 @@ ncio_px_init2(ncio *const nciop, size_t *sizehintp, int isNew)
 		pxp->bf_extent = bufsz;
 		(void) memset(pxp->bf_base, 0, pxp->bf_extent);
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1098,7 +1151,7 @@ static int
 ncio_spx_rel(ncio *const nciop, off_t offset, int rflags)
 {
 	ncio_spx *const pxp = (ncio_spx *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	assert(pxp->bf_offset <= offset);
 	assert(pxp->bf_cnt != 0);
@@ -1147,7 +1200,7 @@ ncio_spx_get(ncio *const nciop,
 		void **const vpp)
 {
 	ncio_spx *const pxp = (ncio_spx *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 #ifdef X_ALIGN
 	size_t rem;
 #endif
@@ -1197,7 +1250,7 @@ ncio_spx_get(ncio *const nciop,
 		 extent,
 		 pxp->bf_base,
 		 &pxp->bf_cnt, &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	pxp->bf_offset = offset;
@@ -1210,7 +1263,7 @@ ncio_spx_get(ncio *const nciop,
 #else
 	*vpp = pxp->bf_base;
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1221,7 +1274,7 @@ strategy(ncio *const nciop, off_t to, off_t offset,
 			size_t extent, int rflags)
 {
 	static ncio_spx pxp[1];
-	int status = ENOERR;
+	int status = NC_NOERR;
 #ifdef X_ALIGN
 	size_t rem;
 #endif
@@ -1271,7 +1324,7 @@ fprintf(stderr, "strategy %ld at %ld to %ld\n",
 		 extent,
 		 pxp->bf_base,
 		 &pxp->bf_cnt, &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	pxp->bf_offset = to; /* TODO: XALIGN */
@@ -1304,7 +1357,7 @@ static int
 ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	off_t lower = from;
 	off_t upper = to;
 	char *base;
@@ -1314,7 +1367,7 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 	rflags &= RGN_NOLOCK; /* filter unwanted flags */
 
 	if(to == from)
-		return ENOERR; /* NOOP */
+		return NC_NOERR; /* NOOP */
 
 	if(to > from)
 	{
@@ -1335,7 +1388,7 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 	status = ncio_spx_get(nciop, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
@@ -1356,7 +1409,7 @@ static int
 ncio_spx_sync(ncio *const nciop)
 {
 	/* NOOP */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 static void
@@ -1404,7 +1457,7 @@ ncio_spx_init2(ncio *const nciop, const size_t *const sizehintp)
 		return ENOMEM;
 	}
 	/* else */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1604,13 +1657,13 @@ posixio_create(const char *path, int ioflags,
 	else
 		status = ncio_px_init2(nciop, sizehintp, 1);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(initialsz != 0)
 	{
 		status = fgrow(fd, (off_t)initialsz);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
@@ -1620,12 +1673,12 @@ posixio_create(const char *path, int ioflags,
 				igeto, igetsz,
                         	RGN_WRITE,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) close(fd);
@@ -1736,7 +1789,7 @@ posixio_open(const char *path,
 	else
 		status = ncio_px_init2(nciop, sizehintp, 0);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(igetsz != 0)
@@ -1745,12 +1798,12 @@ posixio_open(const char *path,
 				igeto, igetsz,
                         	0,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) close(fd); /* assert fd >= 0 */
@@ -1787,7 +1840,7 @@ ncio_px_filesize(ncio *nciop, off_t *filesizep)
 	return errno;
     *filesizep = sb.st_size;
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /*
@@ -1801,7 +1854,7 @@ static int
 ncio_px_pad_length(ncio *nciop, off_t length)
 {
 
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	if(nciop == NULL)
 		return EINVAL;
@@ -1810,13 +1863,13 @@ ncio_px_pad_length(ncio *nciop, off_t length)
 	        return EPERM; /* attempt to write readonly file */
 
 	status = nciop->sync(nciop);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 	        return status;
 
  	status = fgrow2(nciop->fd, length);
- 	if(status != ENOERR)
+ 	if(status != NC_NOERR)
 	        return status;
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1833,7 +1886,7 @@ ncio_px_pad_length(ncio *nciop, off_t length)
 static int
 ncio_px_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(nciop == NULL)
 		return EINVAL;
 	if(nciop->fd > 0) {
@@ -1849,7 +1902,7 @@ ncio_px_close(ncio *nciop, int doUnlink)
 static int
 ncio_spx_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(nciop == NULL)
 		return EINVAL;
 	if(nciop->fd > 0) {
