@@ -303,9 +303,12 @@ v1h_put_NC_string(v1hs *psp, const NC_string *ncstrp)
 static int
 v1h_get_NC_string(v1hs *gsp, NC_string **ncstrpp)
 {
-	int status;
-	size_t padding, nchars = 0;
-	NC_string *ncstrp;
+	int status = 0;
+	size_t nchars = 0;
+	NC_string *ncstrp = NULL;
+#if USE_STRICT_NULL_BYTE_HEADER_PADDING
+        size_t padding = 0;        
+#endif /* USE_STRICT_NULL_BYTE_HEADER_PADDING */
 
 	status = v1h_get_size_t(gsp, &nchars);
 	if(status != NC_NOERR)
@@ -333,17 +336,20 @@ v1h_get_NC_string(v1hs *gsp, NC_string **ncstrpp)
 	if(status != NC_NOERR)
 		goto unwind_alloc;
 
+#if USE_STRICT_NULL_BYTE_HEADER_PADDING
 	padding = _RNDUP(X_SIZEOF_CHAR * ncstrp->nchars, X_ALIGN)
 		- X_SIZEOF_CHAR * ncstrp->nchars;
+
 	if (padding > 0) {
 		/* CDF specification: Header padding uses null (\x00) bytes. */
 		char pad[X_ALIGN-1];
 		memset(pad, 0, X_ALIGN-1);
 		if (memcmp((char*)gsp->pos-padding, pad, padding) != 0) {
 			free_NC_string(ncstrp);
-			return NC_EINVAL;
+			return NC_ENULLPAD;
 		}
 	}
+#endif
 
 	*ncstrpp = ncstrp;
 
@@ -618,11 +624,11 @@ ncmpix_len_nctype(nc_type type) {
 static int
 v1h_put_NC_attrV(v1hs *psp, const NC_attr *attrp)
 {
-	int status;
+	int status = 0;
 	const size_t perchunk =  psp->extent;
 	size_t remaining = attrp->xsz;
 	void *value = attrp->xvalue;
-	size_t nbytes, padding;
+	size_t nbytes = 0, padding = 0;
 
 	assert(psp->extent % X_ALIGN == 0);
 
@@ -641,6 +647,7 @@ v1h_put_NC_attrV(v1hs *psp, const NC_attr *attrp)
 
 	} while(remaining != 0);
 
+
 	padding = attrp->xsz - ncmpix_len_nctype(attrp->type) * attrp->nelems;
 	if (padding > 0) {
 		/* CDF specification: Header padding uses null (\x00) bytes. */
@@ -688,7 +695,10 @@ v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
 	const size_t perchunk =  gsp->extent;
 	size_t remaining = attrp->xsz;
 	void *value = attrp->xvalue;
-	size_t nget, padding;
+	size_t nget;
+#if USE_STRICT_NULL_BYTE_HEADER_PADDING
+	size_t padding;
+#endif /* USE_STRICT_NULL_BYTE_HEADER_PADDING */
 
 	do {
 		nget = MIN(perchunk, remaining);
@@ -706,14 +716,16 @@ v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
 
 	} while(remaining != 0);
 
+#if USE_STRICT_NULL_BYTE_HEADER_PADDING
 	padding = attrp->xsz - ncmpix_len_nctype(attrp->type) * attrp->nelems;
 	if (padding > 0) {
 		/* CDF specification: Header padding uses null (\x00) bytes. */
 		char pad[X_ALIGN-1];
 		memset(pad, 0, X_ALIGN-1);
 		if (memcmp((char*)gsp->pos-padding, pad, (size_t)padding) != 0)
-			return NC_EINVAL;
+			return NC_ENULLPAD;
 	}
+#endif
 
 	return NC_NOERR;
 }
