@@ -1,5 +1,8 @@
 #!/bin/sh
 
+if test "x$srcdir" = x ; then srcdir=`pwd`; fi
+. ../test_common.sh
+
 # This shell runs some parallel I/O tests.
 
 set -e
@@ -33,3 +36,4 @@ echo
 echo "Parallel I/O test for Collective I/O, contributed by HDF Group."
 mpiexec -n 1 ./tst_simplerw_coll_r
 mpiexec -n 2 ./tst_simplerw_coll_r
+mpiexec -n 4 ./tst_simplerw_coll_r
