@@ -1,5 +1,8 @@
 #!/bin/sh
 
+if test "x$srcdir" = x ; then srcdir=`pwd`; fi
+. ../test_common.sh
+
 # This shell just tests the tst_chunks3 program by running it a few
 # times to generate a simple test file. Then it uses ncdump -s to
 # check that the output is what it should be.
@@ -17,8 +20,8 @@ dim2=12
 chunk2=3
 dim3=4
 chunk3=1
-./tst_chunks3 $compress_level $dim1 $chunk1 $dim2 $chunk2 $dim3 $chunk3
-../ncdump/ncdump -n tst_chunks -s tst_chunks3.nc > tst_chunks3.cdl
+${execdir}/tst_chunks3 $compress_level $dim1 $chunk1 $dim2 $chunk2 $dim3 $chunk3
+${NCDUMP} -n tst_chunks -s tst_chunks3.nc > tst_chunks3.cdl
 diff tst_chunks3.cdl ref_chunks1.cdl
 echo '*** SUCCESS!!!'
 
@@ -34,9 +37,9 @@ chunk3=13
 cachesize=10000000
 cachehash=10000
 cachepre=0.0
-./tst_chunks3 $compress_level $dim1 $chunk1 $dim2 $chunk2 $dim3 $chunk3 $cachesize $cachehash $cachepre
-../ncdump/ncdump -n tst_chunks -s -h tst_chunks3.nc > tst_chunks3.cdl
+${execdir}/tst_chunks3 $compress_level $dim1 $chunk1 $dim2 $chunk2 $dim3 $chunk3 $cachesize $cachehash $cachepre
+${NCDUMP} -n tst_chunks -s -h tst_chunks3.nc > tst_chunks3.cdl
 diff tst_chunks3.cdl ref_chunks2.cdl
 echo '*** SUCCESS!!!'
 
-exit 0
\ No newline at end of file
+exit 0
