@@ -16,16 +16,19 @@ int main()
 {
   int  status;
   int  id;
-  int  rh_id, varid, v1, v2, v3, v4;
+  int  v1, v2, v3, v4;
   int  dimids[2];
 
 
   nc_create(FILENAME, NC_CLOBBER, &id);
   nc_redef(id);
 
-  status = nc_def_dim(id, "dim1", 10, &dimids[0]);
-  status = nc_def_var(id, "dim1", NC_FLOAT, 1, dimids, &v1);
-  status = nc_def_var(id, "var1", NC_FLOAT, 1, dimids, &v2);
+  if ((status = nc_def_dim(id, "dim1", 10, &dimids[0])))
+      return status;
+  if ((status = nc_def_var(id, "dim1", NC_FLOAT, 1, dimids, &v1)))
+      return status;
+  if ((status = nc_def_var(id, "var1", NC_FLOAT, 1, dimids, &v2)))
+      return status;
 
   nc_close(id);
 
@@ -35,7 +38,8 @@ int main()
   nc_rename_var(id, v1,"dim_new1");
   nc_rename_dim(id, dimids[0], "dim_new1");
 
-  status = nc_def_dim(id, "dim2", 20, &dimids[1]);
+  if ((status = nc_def_dim(id, "dim2", 20, &dimids[1])))
+      return status;
   nc_def_var(id, "dim2", NC_FLOAT, 1, &dimids[1], &v3);
   nc_def_var(id, "var2", NC_FLOAT, 2, dimids,    &v4);
 
