@@ -32,7 +32,7 @@ BOTHP="[log][show=fetch]"
 BOTHS="noprefetch&fetch=disk"
 
 locreset () {
-    rm -f ./tmp ./errtmp
+    rm -f ./tmp_testurl ./errtmp_testurl
 }
 
 buildurl () {
@@ -58,8 +58,8 @@ echo "***Testing url prefix parameters"
 buildurl $PREFIX ""
 # Invoke ncdump to extract the URL
 echo "command: ${NCDUMP} -h $url"
-${NCDUMP} -h "$url" >./tmp 2> ./errtmp
-if test "x${SHOW}" = x1 ; then cat ./tmp ; fi
+${NCDUMP} -h "$url" >./tmp_testurl 2> ./errtmp_testurl
+if test "x${SHOW}" = x1 ; then cat ./tmp_testurl ; fi
 fi
 
 locreset
@@ -68,8 +68,8 @@ echo "***Testing url suffix parameters"
 buildurl "" $SUFFIX
 # Invoke ncdump to extract the URL
 echo "command: ${NCDUMP} -h $url"
-${NCDUMP} -h "$url" >./tmp  2> ./errtmp
-if test "x${SHOW}" = x1 ; then cat ./tmp ; fi
+${NCDUMP} -h "$url" >./tmp_testurl  2> ./errtmp_testurl
+if test "x${SHOW}" = x1 ; then cat ./tmp_testurl ; fi
 fi
 
 locreset
@@ -78,8 +78,8 @@ echo "***Testing url prefix+suffix parameters"
 buildurl $BOTHP $BOTHS
 # Invoke ncdump to extract the URL
 echo "command: ${NCDUMP} -h $url"
-${NCDUMP} -h "$url" >./tmp 2> ./errtmp
-if test "x${SHOW}" = x1 ; then cat ./tmp ; fi
+${NCDUMP} -h "$url" >./tmp_testurl 2> ./errtmp_testurl
+if test "x${SHOW}" = x1 ; then cat ./tmp_testurl ; fi
 fi
 
 locreset
