@@ -9,7 +9,7 @@
  * definition for va_list from the GNU C compiler.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -113,62 +113,6 @@ char float_att_fmt[] = "%#.NNgf";
 char float_attx_fmt[] = "%#.NNg";
 char double_att_fmt[] = "%#.NNg";
 
-#ifndef HAVE_STRLCAT
-/*	$OpenBSD: strlcat.c,v 1.12 2005/03/30 20:13:52 otto Exp $	*/
-
-/*
- * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
- * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
- * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
- * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
- * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- */
-
-/*
- * Appends src to string dst of size siz (unlike strncat, siz is the
- * full size of dst, not space left).  At most siz-1 characters
- * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
- * Returns strlen(src) + MIN(siz, strlen(initial dst)).
- * If retval >= siz, truncation occurred.
- */
-size_t
-strlcat(char *dst, const char *src, size_t siz)
-{
-	char *d = dst;
-	const char *s = src;
-	size_t n = siz;
-	size_t dlen;
-
-	/* Find the end of dst and adjust bytes left but don't go past end */
-	while (n-- != 0 && *d != '\0')
-		d++;
-	dlen = d - dst;
-	n = siz - dlen;
-
-	if (n == 0)
-		return(dlen + strlen(s));
-	while (*s != '\0') {
-		if (n != 1) {
-			*d++ = *s;
-			n--;
-		}
-		s++;
-	}
-	*d = '\0';
-
-	return(dlen + (s - src));	/* count does not include NUL */
-}
-#endif /* ! HAVE_STRLCAT */
-
-
 /* magic number stored in a safebuf and checked, hoping it will be
  * changed if buffer was overwritten inadvertently */
 #define SAFEBUF_CERT 2147114711
@@ -1313,9 +1257,16 @@ nctime_val_tostring(const ncvar_t *varp, safebuf_t *sfbf, const void *valp) {
     double vv = to_double(varp, valp);
     int separator = formatting_specs.iso_separator ? 'T' : ' ';
     if(isfinite(vv)) {
+	int oldopts = 0;
+	int newopts = 0;
 	int res;
 	sout[0]='"';
+	/* Make nctime dump error messages */
+	oldopts = cdSetErrOpts(0);
+	newopts = oldopts | CU_VERBOSE;
+	cdSetErrOpts(newopts);
 	cdRel2Iso(varp->timeinfo->calendar, varp->timeinfo->units, separator, vv, &sout[1]);
+	cdSetErrOpts(oldopts);
 	res = strlen(sout);
 	sout[res++] = '"';
 	sout[res] = '\0';
