@@ -1,3 +1,4 @@
+#include "config.h"
 #include <nc_tests.h>
 #include "err_macros.h"
 #include <stdio.h>
@@ -10,7 +11,10 @@
 
 #define DIMMAXCLASSIC (NC_MAX_INT - 3)
 #define DIMMAX64OFFSET (NC_MAX_UINT - 3)
+
+#ifdef USE_CDF5
 #define DIMMAX64DATA (NC_MAX_UINT64 - 3)
+#endif
 
 /*
 Test that at least the meta-data works
@@ -30,12 +34,12 @@ main(int argc, char **argv)
     size_t dimsize;
     int dimid;
     int stat = NC_NOERR;
-
+    nc_set_log_level(5);
     printf("\n*** Testing Max Dimension Sizes\n");
 
     printf("\n|size_t|=%lu\n",(unsigned long)sizeof(size_t));
 
-    printf("\n*** Writing Max Dimension Size For NC_CLASSIC\n");
+    printf("\n*** Writing Max Dimension Size (%d) For NC_CLASSIC\n",DIMMAXCLASSIC);
     if ((stat=nc_create(FILECLASSIC, NC_CLOBBER, &ncid))) ERRSTAT(stat);
     dimsize = DIMMAXCLASSIC;
     if ((stat=nc_def_dim(ncid, "testdim", dimsize, &dimid))) ERRSTAT(stat);
@@ -48,10 +52,11 @@ main(int argc, char **argv)
     if(dimsize != DIMMAXCLASSIC) ERR;
     if ((stat=nc_close(ncid))) ERRSTAT(stat);
 
-    printf("\n*** Writing Max Dimension Size For NC_64BIT_OFFSET\n");
+    printf("\n*** Writing Max Dimension Size (%u) For NC_64BIT_OFFSET\n",DIMMAX64OFFSET);
     if ((stat=nc_create(FILE64OFFSET, NC_CLOBBER | NC_64BIT_OFFSET, &ncid))) ERRSTAT(stat);
     dimsize = DIMMAX64OFFSET;
     if ((stat=nc_def_dim(ncid, "testdim", dimsize, &dimid))) ERRSTAT(stat);
+    if ((stat=nc_enddef(ncid))) ERRSTAT(stat);
     if ((stat=nc_close(ncid))) ERRSTAT(stat);
 
     printf("\n*** Reading Max Dimension Size For NC_64BIT_OFFSET\n");
@@ -61,8 +66,9 @@ main(int argc, char **argv)
     if(dimsize != DIMMAX64OFFSET) ERR;
     if ((stat=nc_close(ncid))) ERRSTAT(stat);
 
+#ifdef USE_CDF5
     if(sizeof(size_t) == 8) {
-        printf("\n*** Writing Max Dimension Size For NC_64BIT_DATA\n");
+      printf("\n*** Writing Max Dimension Size (%llu) For NC_64BIT_DATA\n",DIMMAX64DATA);
         if ((stat=nc_create(FILE64DATA, NC_CLOBBER | NC_64BIT_DATA, &ncid))) ERRSTAT(stat);
         dimsize = (size_t)DIMMAX64DATA;
         if ((stat=nc_def_dim(ncid, "testdim", dimsize, &dimid))) ERRSTAT(stat);
@@ -75,6 +81,7 @@ main(int argc, char **argv)
 	if(dimsize != DIMMAX64DATA) ERR;
 	if ((stat=nc_close(ncid))) ERRSTAT(stat);
     }
+#endif /* USE_CDF5 */
 
     SUMMARIZE_ERR;
     FINAL_RESULTS;
