@@ -16,6 +16,10 @@ TESTFILES='tst_comp tst_comp2 tst_enum_data tst_fillbug
  tst_solar_cmp tst_special_atts tst_string_data tst_unicode
  tst_vlen_data'
 
+# Run these programs to create some test files.
+${execdir}/tst_comp2
+${execdir}/tst_compress
+
 echo "*** Testing netCDF-4 features of nccopy on ncdump/*.nc files"
 for i in $TESTFILES ; do
     echo "*** Test nccopy $i.nc copy_of_$i.nc ..."
@@ -86,7 +90,7 @@ echo "*** Test that nccopy -c works as intended for record dimension default (1)
 ${NCGEN} -b -o tst_bug321.nc $srcdir/tst_bug321.cdl
 $NCCOPY -k nc7 -c"lat/2,lon/2" tst_bug321.nc tmp.nc
 ${NCDUMP} -n tst_bug321 tmp.nc > tmp.cdl
-diff $srcdir/tst_bug321.cdl tmp.cdl
+diff -b $srcdir/tst_bug321.cdl tmp.cdl
 # echo "*** Test that nccopy compression with chunking can improve compression"
 rm tst_chunking.nc tmp.nc tmp.cdl tmp-chunked.nc tmp-chunked.cdl tmp-unchunked.nc tmp-unchunked.cdl
 
