@@ -7,7 +7,7 @@
 
 #include        "includes.h"
 #include        "dump.h"
-#include        "offsets.h"
+#include        "ncoffsets.h"
 
 /* Forward*/
 static void computefqns(void);
@@ -422,6 +422,11 @@ processeconstrefs(void)
 {
     unsigned long i;
     /* locate all the datalist and walk them recursively */
+    for(i=0;i<listlength(gattdefs);i++) {
+	Symbol* att = (Symbol*)listget(gattdefs,i);
+	if(att->data != NULL && listlength(att->data) > 0)
+	    processeconstrefsR(att->data);
+    }
     for(i=0;i<listlength(attdefs);i++) {
 	Symbol* att = (Symbol*)listget(attdefs,i);
 	if(att->data != NULL && listlength(att->data) > 0)
@@ -600,6 +605,7 @@ computesize(Symbol* tsym)
 {
     int i;
     int offset = 0;
+    int largealign;
     unsigned long totaldimsize;
     if(tsym->touched) return;
     tsym->touched=1;
@@ -627,29 +633,27 @@ computesize(Symbol* tsym)
 	    break;
 	case NC_COMPOUND: /* keep if all fields are primitive*/
 	    /* First, compute recursively, the size and alignment of fields*/
-	    for(i=0;i<listlength(tsym->subnodes);i++) {
-		Symbol* field = (Symbol*)listget(tsym->subnodes,i);
-		ASSERT(field->subclass == NC_FIELD);
-		computesize(field);
-		/* alignment of struct is same as alignment of first field*/
-		if(i==0) tsym->typ.alignment = field->typ.alignment;
-	    }
-	    /* now compute the size of the compound based on*/
-	    /* what user specified*/
-	    offset = 0;
-            int largealign = 1;
             for(i=0;i<listlength(tsym->subnodes);i++) {
-              Symbol* field = (Symbol*)listget(tsym->subnodes,i);
-              /* only support 'c' alignment for now*/
-              int alignment = field->typ.alignment;
-	      int padding = getpadding(offset,alignment);
-              offset += padding;
-              field->typ.offset = offset;
-              offset += field->typ.size;
-              if (alignment > largealign) {
-                largealign = alignment;
-              }
-	    }
+		Symbol* field = (Symbol*)listget(tsym->subnodes,i);
+                ASSERT(field->subclass == NC_FIELD);
+		computesize(field);
+		if(i==0) tsym->typ.alignment = field->typ.alignment;
+            }
+            /* now compute the size of the compound based on what user specified*/
+            offset = 0;
+            largealign = 1;
+            for(i=0;i<listlength(tsym->subnodes);i++) {
+                Symbol* field = (Symbol*)listget(tsym->subnodes,i);
+                /* only support 'c' alignment for now*/
+                int alignment = field->typ.alignment;
+                int padding = getpadding(offset,alignment);
+                offset += padding;
+                field->typ.offset = offset;
+                offset += field->typ.size;
+                if (alignment > largealign) {
+                    largealign = alignment;
+                }
+            }
 	    tsym->typ.cmpdalign = largealign; /* total structure size alignment */
             offset += (offset % largealign);
 	    tsym->typ.size = offset;
